# coding=utf-8
# Note: this test suite is using pytest instead of the unittest-based scaffolding
# provided by SBE. Hopefully one day all of SBE will follow.

from datetime import datetime, timedelta
from typing import Iterator, Union

import abilian.i18n
import pytest
from abilian.core.signals import activity

from abilian.sbe.app import Application, create_app
from abilian.sbe.apps.communities.common import activity_time_format


@pytest.fixture
def app(config: type) -> Iterator[Union[Iterator, Iterator[Application]]]:
    app = create_app(config)

    # We need some incantations here to make babel work in the test
    babel = abilian.i18n.babel
    babel.locale_selector_func = None

    yield app

    # Signals are globals and apparently need to be cleaned up.
    # At this point, only the "activity" signal seems to have a side effect.
    activity._clear_state()


def test_activity_time_format(app: Application) -> None:
    # We need the app context because of Babel.

    with app.app_context():
        then = datetime(2017, 1, 1, 12, 0, 0)

        now = then + timedelta(0, 5)
        assert activity_time_format(then, now) == "5s"

        now = then + timedelta(0, 5 * 60)
        assert activity_time_format(then, now) == "5m"

        now = then + timedelta(0, 5 * 60 * 60)
        assert activity_time_format(then, now) == "5h"

        now = then + timedelta(1, 5)
        assert activity_time_format(then, now) == "1d"

        now = then + timedelta(60, 5)
        assert activity_time_format(then, now) == "Jan 1"

        now = then + timedelta(365 + 60, 5)
        assert activity_time_format(then, now) == "Jan 2017"



# coding=utf-8
import re

from abilian.core.models.subjects import User
from abilian.testing.util import client_login
from flask import g, url_for
from pytest import fixture
from toolz import first

from abilian.sbe.apps.communities.models import READER, Community
from abilian.sbe.apps.wiki import views
from abilian.sbe.apps.wiki.models import WikiPage


@fixture
def user1(db):
    user = User(email="user_1@example.com", password="azerty", can_login=True)
    db.session.add(user)
    return user


@fixture
def user2(db):
    user = User(email="user_2@example.com", password="azerty", can_login=True)
    db.session.add(user)
    return user


@fixture
def user3(db):
    user = User(email="user_3@example.com", password="azerty", can_login=True)
    db.session.add(user)
    return user


@fixture
def community1(db, user1):
    community = Community(name="Community 1")
    community.set_membership(user1, READER)
    db.session.add(community)
    return community


@fixture
def community2(db, user2):
    community = Community(name="Community 2")
    community.set_membership(user2, READER)
    db.session.add(community)
    return community


def test_home(client, community1, user1, req_ctx):
    with client_login(client, user1):
        response = client.get(url_for("wiki.index", community_id=community1.slug))
        assert response.status_code == 302

        response = client.get(
            url_for("wiki.page", title="Home", community_id=community1.slug)
        )
        assert response.status_code == 200


def test_create_page_initial_form(client, community1, user1, req_ctx):
    with client:
        with client_login(client, user1):
            g.community = community1
            view = views.PageCreate()
            view.prepare_args([], {})
            form = view.form
            assert form["last_revision_id"].data is None


def test_wiki_indexed(
    community1, community2, admin_user, user1, user2, user3, app, db, client, req_ctx
):
    SERVICES = ("security", "indexing")
    for svc in SERVICES:
        svc = app.services[svc]
        if not svc.running:
            svc.start()

    svc = app.services["indexing"]

    with client:
        with client_login(client, admin_user):
            page1 = WikiPage(title="Community 1", community=community1)
            db.session.add(page1)

            page2 = WikiPage(title="Community 2: other", community=community2)
            db.session.add(page2)

            db.session.commit()

        obj_types = (WikiPage.entity_type,)
        with client_login(client, user3):
            res = svc.search("community", object_types=obj_types)
            assert len(res) == 0

        with client_login(client, user1):
            res = svc.search("community", object_types=obj_types)
            assert len(res) == 1

            hit = res[0]
            assert hit["object_key"] == page1.object_key

        with client_login(client, user2):
            res = svc.search("community", object_types=obj_types)
            assert len(res) == 1

            hit = res[0]
            assert hit["object_key"] == page2.object_key


def test_create_page(community1, app, admin_user, client, req_ctx):
    community = community1

    with client:
        with client_login(client, admin_user):
            url = url_for("wiki.page_new", community_id=community.slug)
            url += "?title=Some+page+name"
            response = client.get(url)
            assert response.status_code == 200
            # make sure the title is filled when comming from wikilink
            assert 'value="Some page name"' in response.get_data(as_text=True)

            title = "Some page"
            body = "LuuP3jai"
            url = url_for("wiki.page_new", community_id=community.slug)
            data = {"title": title, "body_src": body, "__action": "create"}
            response = client.post(url, data=data)
            assert response.status_code == 302

            url = url_for("wiki.page", community_id=community.slug, title=title)
            response = client.get(url)
            assert response.status_code == 200
            assert body in response.get_data(as_text=True)

            # edit
            url = url_for("wiki.page_edit", community_id=community.slug, title=title)
            response = client.get(url)
            assert response.status_code == 200

            # Slightly hackish way to get the page_id
            line = first(
                l
                for l in response.get_data(as_text=True).split("\n")
                if 'name="page_id"' in l
            )
            m = re.search('value="(.*?)"', line)
            page_id = int(m.group(1))

            url = url_for("wiki.page_changes", community_id=community.slug, title=title)
            response = client.get(url)
            assert response.status_code == 200

            url = url_for("wiki.page_source", community_id=community.slug, title=title)
            response = client.get(url)
            assert response.status_code == 200

            url = url_for("wiki.page_edit", community_id=community.slug)
            data = {"title": title, "page_id": page_id, "body_src": "abc def"}
            data["__action"] = "edit"
            response = client.post(url, data=data)
            assert response.status_code == 302

            url = url_for(
                "wiki.page_compare",
                rev0="on",
                rev1="on",
                community_id=community.slug,
                title=title,
            )
            response = client.get(url)
            assert response.status_code == 200



# coding=utf-8
from unittest import mock
from urllib.parse import quote_plus

import pytest
from abilian.testing.util import client_login
from flask_login import current_user
from markdown import Markdown

from abilian.sbe.apps.wiki.markup import SBEWikiLinkExtension
from abilian.sbe.apps.wiki.models import WikiPage


@pytest.mark.parametrize("text", ["TOTO", "x 123", "/#$", "/*€("])
def test_wikilink_extension(text, db, req_ctx):
    qtext = str(quote_plus(text.encode("utf-8")))
    wikilink = "[[" + text + "]]"

    def build_url(label: str, base: str, end: str) -> str:
        print("build_url called")
        return "/?title=" + quote_plus(label.encode("utf-8")) + end

    extension = SBEWikiLinkExtension(build_url=build_url)
    ctx = {
        "extensions": [extension, "markdown.extensions.toc"],
        "output_format": "html5",
    }
    md = Markdown(**ctx)

    def check(page_exists: bool) -> None:
        page_exists_mock = mock.MagicMock(return_value=page_exists)
        with mock.patch("abilian.sbe.apps.wiki.markup.page_exists", page_exists_mock):
            result = md.convert(wikilink)

        if page_exists:
            expected_tpl = (
                '<p><a class="wikilink" href="/?title={href}/">{text}</a></p>'
            )
        else:
            expected_tpl = (
                '<p><a class="wikilink new" href="/?title={href}/">{text}</a></p>'
            )

        expected = expected_tpl.format(href=qtext, text=text)
        assert expected == result

    check(True)
    check(False)


def test_new_page(user, client, req_ctx):
    with client:
        with client_login(client, user):
            print(current_user)
            page = WikiPage(title="Some page", body_src="abc")
            assert page.title == "Some page"
            assert page.name == "Some page"
            assert page.body_src == "abc"
            assert page.body_html == "<p>abc</p>"
            assert len(page.revisions) == 1

            revision = page.revisions[0]
            assert revision.number == 0
            assert revision.author == user


def test_rename_page(user, req_ctx):
    page = WikiPage(title="Some page", body_src="abc")
    assert page.title == "Some page"
    assert page.name == "Some page"

    page.title = "Title Renamed"
    assert page.title == "Title Renamed"
    assert page.name == "Title Renamed"

    page.name = "Name"
    assert page.title == "Name"
    assert page.name == "Name"


def test_create_revision(user, client, req_ctx):
    with client:
        with client_login(client, user):
            page = WikiPage("abc")
            page.create_revision("def", "page updated")

            assert len(page.revisions) == 2
            last_revision = page.revisions[1]
            assert last_revision.number == 1
            assert last_revision.author == user



# coding=utf-8
""""""
from datetime import datetime
from typing import Any, Optional

from abilian.core.entities import Entity, db
from abilian.core.models import SEARCHABLE
from abilian.core.models.subjects import User
from flask_login import current_user
from sqlalchemy import Column, DateTime, ForeignKey, Integer, Unicode, \
    UnicodeText, UniqueConstraint
from sqlalchemy.event import listens_for
from sqlalchemy.ext.hybrid import hybrid_property
from sqlalchemy.orm import backref, relationship

from abilian.sbe.apps.communities.models import Community, CommunityIdColumn, \
    community_content
from abilian.sbe.apps.documents.models import BaseContent

from . import markup

__all__ = ["WikiPage", "WikiPageAttachment", "WikiPageRevision"]


@community_content
class WikiPage(Entity):
    __tablename__ = "wiki_page"

    # : The title for this page
    _title = Column("title", Unicode(200), nullable=False, index=True)

    community_id = CommunityIdColumn()
    #: The community this page belongs to
    community = relationship(
        Community,
        primaryjoin=(community_id == Community.id),
        backref=backref("wiki", cascade="all, delete-orphan"),
    )

    #: The body, using some markup language (Markdown for now)
    body_src = Column(
        UnicodeText,
        default="",
        nullable=False,
        info=SEARCHABLE | {"index_to": ("text",)},
    )

    __table_args__ = (UniqueConstraint("title", "community_id"),)

    def __init__(self, title="", body_src="", message="", *args, **kwargs):
        Entity.__init__(self, *args, **kwargs)
        self.title = title
        self.create_revision(body_src, message)

    # title is defined has an hybrid property to allow name <-> title sync
    # (2 way)
    @hybrid_property
    def title(self: str) -> str:
        return self._title

    @title.setter
    def title(self, title: str) -> None:
        # set title before setting name, so that we don't enter an infinite loop
        # with _wiki_sync_name_title
        self._title = title
        if self.name != title:
            self.name = title

    def create_revision(self, body_src: str, message: str = "") -> None:
        revision = WikiPageRevision()
        if self.revisions:
            revision.number = max(r.number for r in self.revisions) + 1
        else:
            revision.number = 0
        self.body_src = revision.body_src = body_src
        revision.message = message
        revision.author = current_user
        revision.page = self

    @property
    def last_revision(self):
        return (
            WikiPageRevision.query.filter(WikiPageRevision.page == self)
            .order_by(WikiPageRevision.number.desc())
            .first()
        )

    @property
    def body_html(self) -> str:
        html = markup.convert(self, self.body_src)
        return html

        # TODO: remove Javascript from content
        # return bleach.clean(html, strip=True)


@listens_for(WikiPage.name, "set", active_history=True)
def _wiki_sync_name_title(entity, new_value, old_value, initiator):
    """Synchronize wikipage name -> title.

    wikipage.title -> name is done via hybrid_property, avoiding
    infinite loop (since "set" is received before attribute has received
    value)
    """
    if entity.title != new_value:
        entity.title = new_value
    return new_value


class WikiPageRevision(db.Model):
    __tablename__ = "wiki_page_revision"
    __table_args__ = (UniqueConstraint("page_id", "number"),)

    # : Some primary key just in case
    id = Column(Integer, primary_key=True)

    #: Date and time this revision was created
    created_at = Column(DateTime, default=datetime.utcnow)

    #: The page this revision belongs to
    page = relationship(WikiPage, backref="revisions")
    page_id = Column(ForeignKey(WikiPage.id))

    #: The revision number
    number = Column(Integer, nullable=False)

    #: The body, using some markup language (Markdown for now)
    body_src = Column(UnicodeText, default="", nullable=False)

    #: Commit message
    message = Column(UnicodeText, default="", nullable=False)

    #: The author of this revision
    author = relationship(User)
    author_id = Column(ForeignKey(User.id))


class WikiPageAttachment(BaseContent):
    __tablename__ = None  # type: str
    __indexable__ = False
    __mapper_args__ = {"polymorphic_identity": "wikipage_attachment"}
    sbe_type = "wikipage:attachment"

    _wikipage_id = Column(Integer, ForeignKey(WikiPage.id), nullable=True)
    wikipage = relationship(
        WikiPage,
        primaryjoin=(_wikipage_id == WikiPage.id),
        backref=backref(
            "attachments",
            lazy="select",
            order_by="WikiPageAttachment.name",
            cascade="all, delete-orphan",
        ),
    )



# coding=utf-8
"""Calendar module."""


def register_plugin(app):
    from .views import blueprint
    from .actions import register_actions

    blueprint.record_once(register_actions)
    app.register_blueprint(blueprint)



# -*- coding: utf-8 -*-
"""Document management blueprint."""
from typing import Dict

from abilian.i18n import _l
from abilian.web.action import Endpoint
from abilian.web.nav import BreadcrumbItem
from flask import g

from abilian.sbe.apps.communities.blueprint import Blueprint
from abilian.sbe.apps.communities.security import is_manager
from abilian.sbe.apps.documents.actions import register_actions

__all__ = ["blueprint"]

blueprint = Blueprint(
    "documents", __name__, url_prefix="/docs", template_folder="../templates"
)
route = blueprint.route
blueprint.record_once(register_actions)


@blueprint.url_value_preprocessor
def init_document_values(endpoint: str, values: Dict[str, int]) -> None:
    g.current_tab = "documents"
    g.is_manager = is_manager()

    g.breadcrumb.append(
        BreadcrumbItem(
            label=_l("Documents"),
            url=Endpoint("documents.index", community_id=g.community.slug),
        )
    )



# coding=utf-8
"""Lightweight integration and denormalisation using events (signals)."""

from typing import Any, Union

from abilian.core.models.subjects import User
from abilian.core.signals import activity
from blinker import ANY
from werkzeug.local import LocalProxy

from abilian.sbe.apps.communities.presenters import CommunityPresenter
from abilian.sbe.apps.documents.models import Document
from abilian.sbe.apps.wiki.models import WikiPage

from .models import Community


@activity.connect_via(ANY)
def update_community(
    sender: Any,
    verb: str,
    actor: Union[User, LocalProxy],
    object: Union[Community, Document, WikiPage],
    target: Union[None, Community, CommunityPresenter] = None,
) -> None:
    if isinstance(object, Community):
        object.touch()
        return

    if isinstance(target, Community):
        community = target
        community.touch()

        if isinstance(object, Document):
            if verb == "post":
                community.document_count += 1
            elif verb == "delete":
                community.document_count -= 1



# coding=utf-8
from pytest import mark

from abilian.sbe.apps.documents.cmis.renderer import Feed, to_xml
from abilian.sbe.apps.documents.models import Document, Folder


@mark.usefixtures("app_context")
def test_folder_renderer() -> None:
    folder = Folder(name="tototiti")
    result = to_xml(folder)
    assert "tototiti" in result
    assert "cmis:folder" in result


@mark.usefixtures("app_context")
def test_document_renderer() -> None:
    document = Document(name="tototiti")
    result = to_xml(document)
    assert "tototiti" in result
    assert "cmis:document" in result
    assert "<?xml" in result

    result = to_xml(document, no_xml_header=True)
    assert "<?xml" not in result


@mark.usefixtures("app_context")
def test_feed_renderer() -> None:
    folder = Folder(title="Toto Titi")
    document = Document(title="tatatutu")
    feed = Feed(folder, [document])
    result = feed.to_xml()

    assert "Toto Titi" in result
    assert "tatatutu" in result



# coding=utf-8
"""Forum views."""
from datetime import date, datetime

from abilian.i18n import _l
from abilian.web import url_for, views
from abilian.web.action import ButtonAction
from flask import g, render_template
from toolz import groupby

from ..communities.blueprint import Blueprint
from ..communities.views import default_view_kw
from .forms import EventForm
from .models import Event

blueprint = Blueprint(
    "calendar", __name__, url_prefix="/calendar", template_folder="templates"
)
route = blueprint.route


@route("/")
def index():
    events = Event.query.filter(Event.end > datetime.now()).order_by(Event.start).all()

    def get_month(event):
        year = event.start.year
        month = event.start.month
        return date(year, month, 1)

    groups = sorted(groupby(get_month, events).items())
    ctx = {"groups": groups}
    return render_template("calendar/index.html", **ctx)


@route("/archives/")
def archives():
    events = (
        Event.query.filter(Event.end <= datetime.now())
        .order_by(Event.start.desc())
        .all()
    )

    def get_month(event):
        year = event.start.year
        month = event.start.month
        return date(year, month, 1)

    groups = sorted(groupby(get_month, events).items(), reverse=True)
    ctx = {"groups": groups}
    return render_template("calendar/archives.html", **ctx)


class BaseEventView:
    Model = Event
    Form = EventForm
    pk = "event_id"
    base_template = "community/_base.html"

    def index_url(self):
        return url_for(".index", community_id=g.community.slug)

    def view_url(self):
        return url_for(self.obj)


class EventView(BaseEventView, views.ObjectView):
    methods = ["GET", "HEAD"]
    Form = EventForm
    template = "calendar/event.html"

    @property
    def template_kwargs(self):
        kw = super().template_kwargs
        kw["event"] = self.obj
        return kw


event_view = EventView.as_view("event")
views.default_view(blueprint, Event, "event_id", kw_func=default_view_kw)(event_view)
route("/<int:event_id>/")(event_view)


class EventCreateView(BaseEventView, views.ObjectCreate):
    POST_BUTTON = ButtonAction(
        "form", "create", btn_class="primary", title=_l("Post this event")
    )

    title = _l("New event")

    def after_populate_obj(self):
        if self.obj.community is None:
            self.obj.community = g.community._model

    def get_form_buttons(self, *args, **kwargs):
        return [self.POST_BUTTON, views.object.CANCEL_BUTTON]

    @property
    def activity_target(self):
        return self.obj.community


event_create_view = EventCreateView.as_view("new_event", view_endpoint=".event")
route("/new_event/")(event_create_view)


class EventEditView(BaseEventView, views.ObjectEdit):
    POST_BUTTON = ButtonAction(
        "form", "create", btn_class="primary", title=_l("Post this event")
    )

    title = _l("Edit event")


event_edit_view = EventEditView.as_view("event_edit", view_endpoint=".event")
route("/<int:event_id>/edit")(event_edit_view)



# coding=utf-8
"""Register extensions as a plugin.

NOTE: panels are currently loaded and registered manually. This may change
in the future.
"""


def register_plugin(app):
    from .main import blueprint

    app.register_blueprint(blueprint)



# coding=utf-8
""""""
from datetime import datetime
from typing import Any, Optional
from urllib.parse import quote

import sqlalchemy as sa
import sqlalchemy.orm
from abilian.core.extensions import db, mail
from abilian.core.signals import activity
from abilian.core.util import unwrap
from abilian.i18n import _, render_template_i18n
from abilian.services import audit_service
from abilian.services.conversion import converter
from abilian.services.image import FIT, resize
from abilian.services.viewtracker import viewtracker
from abilian.web import csrf, url_for
from abilian.web.action import actions
from abilian.web.frontend import add_to_recent_items
from abilian.web.views import default_view
from flask import current_app, flash, g, make_response, redirect, \
    render_template, request
from flask_login import current_user
from flask_mail import Message
from werkzeug.exceptions import BadRequest, NotFound
from werkzeug.wrappers.response import Response

from abilian.sbe.apps.communities.common import object_viewers
from abilian.sbe.apps.communities.views import default_view_kw
from abilian.sbe.apps.documents.models import Document
from abilian.sbe.apps.documents.repository import repository
from abilian.sbe.apps.documents.tasks import convert_document_content, \
    preview_document

from .util import breadcrumbs_for, check_manage_access, check_read_access, \
    check_write_access, edit_object, get_document, get_folder, match
from .views import blueprint

route = blueprint.route

MAX_PREVIEW_SIZE = 1000

__all__ = ()


@default_view(blueprint, Document, id_attr="doc_id", kw_func=default_view_kw)
@route("/doc/<int:doc_id>")
def document_view(doc_id: int) -> str:
    doc = get_document(doc_id)
    check_read_access(doc)
    doc.ensure_antivirus_scheduled()
    db.session.commit()

    bc = breadcrumbs_for(doc)
    actions.context["object"] = doc

    if doc.content_type.startswith("image/"):
        add_to_recent_items(doc, "image")
    else:
        add_to_recent_items(doc, "document")

    has_preview = doc.has_preview()
    audit_entries = audit_service.entries_for(doc)
    viewtracker.record_hit(entity=doc, user=current_user)

    ctx = {
        "doc": doc,
        "audit_entries": audit_entries,
        "breadcrumbs": bc,
        "folder": doc.parent,
        "has_preview": has_preview,
        "viewers": object_viewers(doc),
    }
    return render_template("documents/document.html", **ctx)


#
# Actions on documents
#
@route("/doc/<int:doc_id>/", methods=["POST"])
@route("/doc/<int:doc_id>/<int:folder_id>/", methods=["POST"])
@csrf.protect
# TODO: URL doesn't seem right
def document_edit(doc_id, folder_id=None):
    doc = get_document(doc_id)
    if folder_id:
        folder = get_folder(folder_id)
    else:
        folder = None
    check_write_access(doc)

    changed = edit_object(doc)

    if changed:
        db.session.commit()
        flash(_("Document properties successfully edited."), "success")
    else:
        flash(_("You didn't change any property."), "success")

    if folder:
        return redirect(url_for(folder))
    else:
        return redirect(url_for(doc))


@route("/doc/<int:doc_id>/viewers", methods=["GET"])
def document_viewers(doc_id):
    doc = get_document(doc_id)
    check_read_access(doc)
    doc.ensure_antivirus_scheduled()
    # db.session.commit()

    bc = breadcrumbs_for(doc)
    actions.context["object"] = doc
    """if doc.content_type.startswith("image/"):
        add_to_recent_items(doc, "image")
    else:
        add_to_recent_items(doc, "document")"""

    has_preview = doc.has_preview()
    audit_entries = audit_service.entries_for(doc)

    ctx = {
        "doc": doc,
        "audit_entries": audit_entries,
        "breadcrumbs": bc,
        "folder": doc.parent,
        "has_preview": has_preview,
        "viewers": object_viewers(doc),
    }
    return render_template("documents/document_viewers.html", **ctx)


@route("/doc/<int:doc_id>/delete", methods=["POST"])
@csrf.protect
def document_delete(doc_id: int) -> Response:
    doc = get_document(doc_id)
    check_write_access(doc)

    parent_folder = doc.parent
    repository.delete_object(doc)
    db.session.commit()

    flash(_("File successfully deleted."), "success")
    return redirect(url_for(parent_folder))


@route("/doc/<int:doc_id>/upload", methods=["POST"])
@csrf.protect
def document_upload(doc_id):
    doc = get_document(doc_id)
    check_write_access(doc)

    fd = request.files["file"]
    doc.set_content(fd.read(), fd.content_type)
    del doc.lock

    self = unwrap(current_app)
    activity.send(self, actor=current_user, verb="update", object=doc)
    db.session.commit()
    flash(_("New version successfully uploaded"), "success")
    return redirect(url_for(doc))


@route("/doc/<int:doc_id>/download")
def document_download(doc_id: int, attach: bool = False) -> Response:
    """Download the file content."""
    doc = get_document(doc_id)

    response = make_response(doc.content)
    response.headers["content-length"] = doc.content_length
    response.headers["content-type"] = doc.content_type

    if not attach:
        attach = request.args.get("attach", False)

    if attach or not match(
        doc.content_type, ("text/plain", "application/pdf", "image/*")
    ):
        # Note: we omit text/html for security reasons.
        quoted_filename = quote(doc.title.encode("utf8"))
        response.headers["content-disposition"] = 'attachment;filename="{}"'.format(
            quoted_filename
        )

    return response


@route("/doc/<int:doc_id>/checkin_checkout", methods=["POST"])
def checkin_checkout(doc_id):
    doc = get_document(doc_id)
    action = request.form.get("action")

    if action not in ("checkout", "lock", "unlock"):
        raise BadRequest(f"Unknown action: {action!r}")

    session = sa.orm.object_session(doc)

    if action in ("lock", "checkout"):
        doc.lock = current_user
        d = doc.updated_at
        # prevent change of last modification date
        doc.updated_at = datetime.utcnow()
        session.flush()
        doc.updated_at = d
        session.commit()

        if action == "lock":
            return redirect(url_for(doc))
        elif action == "checkout":
            return document_download(doc_id, attach=True)

    if action == "unlock":
        del doc.lock
        d = doc.updated_at
        # prevent change of last modification date
        doc.updated_at = datetime.utcnow()
        session.flush()
        doc.updated_at = d
        session.commit()
        return redirect(url_for(doc))


def preview_missing_image():
    response = redirect(
        url_for("abilian_sbe_static", filename="images/preview_missing.png")
    )
    response.headers["Cache-Control"] = "no-cache"
    return response


@route("/doc/<int:doc_id>/preview_image")
def document_preview_image(doc_id: int) -> Response:
    """Returns a preview (image) for the file given by its id."""

    doc = get_document(doc_id)

    if not doc.antivirus_ok:
        return preview_missing_image()

    size = int(request.args.get("size", 0))

    # Just in case
    if size > MAX_PREVIEW_SIZE:
        size = MAX_PREVIEW_SIZE

    # compute image if size != standard document size
    get_image = converter.get_image if size == doc.preview_size else converter.to_image

    content_type = "image/jpeg"

    if doc.content_type.startswith("image/svg"):
        image = doc.content
        content_type = doc.content_type
    elif doc.content_type.startswith("image/"):
        image = doc.content
        if size:
            image = resize(image, size, size, mode=FIT)
    else:
        page = int(request.args.get("page", 0))
        try:
            image = get_image(doc.digest, doc.content, doc.content_type, page, size)
        except BaseException:
            # TODO: use generic "conversion failed" image
            image = ""

    if not image:
        return preview_missing_image()

    response = make_response(image)
    response.headers["content-type"] = content_type
    return response


@route("/doc/<int:doc_id>/refresh_preview")
def refresh_preview(doc_id):
    """Force to compute a new preview."""
    doc = get_document(doc_id)
    if not doc:
        raise NotFound()

    ct = doc.find_content_type(doc.content_type)
    if ct != doc.content_type:
        doc.content_type = ct
        db.session.commit()

    check_manage_access(doc)
    convert_document_content.apply([doc_id])
    preview_document.apply([doc_id])
    return redirect(url_for(doc))


@route("/doc/<int:doc_id>/send", methods=["POST"])
@csrf.protect
def document_send(doc_id: int) -> Response:
    doc = get_document(doc_id)

    recipient = request.form.get("recipient")
    user_msg = request.form.get("message")

    site_name = "[{}] ".format(current_app.config["SITE_NAME"])
    sender_name = current_user.name
    subject = site_name + _("{sender} sent you a file").format(sender=sender_name)
    msg = Message(subject)
    msg.sender = current_user.email
    msg.recipients = [recipient]
    msg.body = render_template_i18n(
        "documents/mail_file_sent.txt",
        sender_name=sender_name,
        message=user_msg,
        document_url=url_for(doc),
        filename=doc.title,
    )

    filename = doc.title
    msg.attach(filename, doc.content_type, doc.content)

    mail.send(msg)
    flash(_("Email successfully sent"), "success")

    return redirect(url_for(doc))


@route("/doc/<int:doc_id>/preview")
def document_preview(doc_id):
    doc = get_document(doc_id)
    if not doc.antivirus_ok:
        return "Waiting for antivirus to finish"

    if doc.content_type == "application/pdf":
        return redirect(
            url_for(".document_view_pdf", community_id=g.community.slug, doc_id=doc.id)
        )

    else:
        return redirect(
            url_for(".document_download", community_id=g.community.slug, doc_id=doc.id)
        )


@route("/doc/<int:doc_id>/view_pdf")
def document_view_pdf(doc_id):
    doc = get_document(doc_id)
    if not doc.antivirus_ok:
        return "Waiting for antivirus to finish"

    return render_template(
        "documents/view_pdf.html",
        pdf_url=url_for(
            ".document_download", community_id=g.community.slug, doc_id=doc.id
        ),
    )


#
# Tagging (currently not used!)
#
# @route("/tag")
# def tag():
#     tag = request.args.get("tag")
#     if not tag:
#         return redirect("/dm/")
#
#     bc = [dict(path="/", label="Home"), dict(path="/dm/", label="DM")]
#     bc += [dict(path=request.path, label="Filter by tag")]
#     # TODO ...
#     docs = Document.query.filter(Document.tags.like("%" + tag + "%"))
#     docs = list(docs.all())
#     docs = [f for f in docs if tag in f.tags.split(",")]
#     title = "Files filtered by tag: %s" % tag
#     return render_template("dm/home.html", title=title, breadcrumbs=bc,
#                            files=docs)
#
#
# @route("/<int:file_id>/tag", methods=['POST'])
# def tag_post(file_id):
#     doc = get_document(file_id)
#     tags = request.form.get("tags")
#
#     doc.tags = tags
#     self = unwrap(current_app)
#     activity.send(self, actor=g.user, verb="tag", object=doc)
#
#     db.session.commit()
#
#     flash("Tags successfully successfully updated", "success")
#     return redirect(url_for(".document_view", doc_id=doc.id))



# coding=utf-8
""""""
from abilian.core.entities import SEARCHABLE, Entity
from sqlalchemy import Column, DateTime, Unicode
from sqlalchemy.event import listens_for
from sqlalchemy.orm import backref, relationship

from abilian.sbe.apps.communities.models import Community, CommunityIdColumn, \
    community_content


@community_content
class Event(Entity):
    __tablename__ = "sbe_event"

    community_id = CommunityIdColumn()
    #: The community this event belongs to
    community = relationship(
        Community,
        primaryjoin=(community_id == Community.id),
        backref=backref("events", cascade="all, delete-orphan"),
    )

    title = Column(Unicode, nullable=False, default="", info=SEARCHABLE)

    description = Column(Unicode, nullable=False, default="", info=SEARCHABLE)

    location = Column(Unicode, nullable=False, default="", info=SEARCHABLE)

    start = Column(DateTime, nullable=False)
    end = Column(DateTime)

    url = Column(Unicode, nullable=False, default="")


@listens_for(Event.title, "set", active_history=True)
def _event_sync_name_title(entity, new_value, old_value, initiator):
    if entity.name != new_value:
        entity.name = new_value
    return new_value



# coding=utf-8

from abilian.core.entities import all_entity_classes
from abilian.core.models.subjects import Group, User

from abilian.sbe.apps.social.models import Message, PrivateMessage


def check_editable(object):
    if hasattr(object, "__editable__"):
        for k in object.__editable__:
            assert hasattr(object, k)


def test_user() -> None:
    user = User(
        first_name="John",
        last_name="Test User",
        email="test@example.com",
        password="toto",
    )
    check_editable(user)

    assert "John Test User" == user.name
    assert "John Test User" == str(user)
    # self.assertEquals(len(user.messages), 0)


def test_user_follow() -> None:
    # id is provided by DB (which is not used in this test), and is required for
    # having in (user1 != user2) == True
    user1 = User(
        id=1,
        first_name="John",
        last_name="Test User 1",
        email="test1@example.com",
        password="toto",
    )
    user2 = User(
        id=2,
        first_name="Joe",
        last_name="Test User 2",
        email="test2@example.com",
        password="toto",
    )

    assert len(user1.followers) == 0
    assert len(user1.followees) == 0
    assert len(user2.followers) == 0
    assert len(user2.followees) == 0

    user1.follow(user2)

    assert user1.is_following(user2)
    assert len(user2.followers) == 1
    assert len(user1.followees) == 1
    assert len(user2.followees) == 0
    assert len(user1.followers) == 0
    assert user2 in user1.followees

    user1.unfollow(user2)

    assert not user1.is_following(user2)
    assert len(user1.followers) == 0
    assert len(user1.followees) == 0
    assert len(user2.followers) == 0
    assert len(user2.followees) == 0
    assert user2 not in user1.followers


def test_group() -> None:
    user = User(
        first_name="John",
        last_name="Test User",
        email="test@example.com",
        password="toto",
    )
    group = Group(name="Group 1")

    user.join(group)

    assert user.is_member_of(group)
    assert len(group.members) == 1
    assert len(user.groups) == 1
    assert group.members == {user}
    assert user.groups == {group}

    user.leave(group)

    assert not user.is_member_of(group)
    assert len(group.members) == 0
    assert len(user.groups) == 0


def test_private_message() -> None:
    pm = PrivateMessage(creator_id=0, recipient_id=0)
    check_editable(pm)


# TODO: implement status updates (aka messages)
# def test_status_update(db):
#     user = User(
#         first_name="John",
#         last_name="Test User",
#         email="test@example.com",
#         password="toto",
#     )
#     db.session.commit()
#     assert len(user.messages) == 0
#
#     message = Message()
#     message.author = user
#     check_editable(message)
#
#     db.session.commit()
#     assert len(user.messages) == 1
#     assert user.messages[0] == message
#     assert message.author_id == user.uid


def test_tags() -> None:
    m = Message(content="abc #123 #cde #voilà_l_été #789")
    assert m.tags == ["123", "cde", "voilà_l_été", "789"]


def test_get_all_entity_classes() -> None:
    classes = all_entity_classes()
    assert Message in classes
    assert PrivateMessage in classes
